
#include <windows.h>
#include <stdio.h>

double overhead(unsigned long n)
{
    unsigned long i;
    __int64 hz, cnt, cnt0;
    QueryPerformanceFrequency((LARGE_INTEGER*)&hz);
    QueryPerformanceCounter((LARGE_INTEGER*)&cnt0);
    for (i=0;i<n;i++) {
	QueryPerformanceCounter((LARGE_INTEGER*)&cnt);
    }
    return (double)(cnt - cnt0)/(double)hz;
}

extern unsigned long genrand_int32(void);
extern unsigned long array_genrand_int32(unsigned long n, unsigned long array[]);
extern void init_by_array(unsigned long init_key[], int key_length);

#define CHUNK 10000000
__declspec(align(16)) static unsigned long arr [CHUNK];

double test_mt19937_array(void)
{
    unsigned long i, tot=0;
    unsigned long init[4]={0x123, 0x234, 0x345, 0x456}, length=4;
    double sec, oh;
    __int64 hz, cnt, cnt0, stop;
    init_by_array(init, length);
    QueryPerformanceFrequency((LARGE_INTEGER*)&hz);
    stop = 5 * hz;
    QueryPerformanceCounter((LARGE_INTEGER*)&cnt0);
    for (;;) {
        array_genrand_int32(CHUNK, arr);
        tot += CHUNK;
	QueryPerformanceCounter((LARGE_INTEGER*)&cnt);
        if (cnt - cnt0 >= stop) break;
    }
    sec = (double)(cnt - cnt0)/(double)hz;
    oh = overhead(tot/CHUNK);
    return (double)tot/(sec-oh);
}

double test_mt19937(void)
{
    unsigned long i, tot=0;
    unsigned long init[4]={0x123, 0x234, 0x345, 0x456}, length=4, x;
    double sec, oh;
    __int64 hz, cnt, cnt0, stop;
    init_by_array(init, length);
    QueryPerformanceFrequency((LARGE_INTEGER*)&hz);
    stop = 5 * hz;
    QueryPerformanceCounter((LARGE_INTEGER*)&cnt0);
    for (;;) {
        for (i=0; i<CHUNK; i++) x = genrand_int32();
        tot += CHUNK;
	QueryPerformanceCounter((LARGE_INTEGER*)&cnt);
        if (cnt - cnt0 >= stop) break;
    }
    sec = (double)(cnt - cnt0)/(double)hz;
    oh = overhead(tot/CHUNK);
    return (double)tot/(sec-oh);
}

extern void init_kiss4691(void);
extern unsigned long kiss4691(void);
extern void array_kiss4691(unsigned long n, unsigned long array[]);

double test_kiss4691(void)
{
    unsigned long i,x,tot=0;
    double sec, oh;
    __int64 hz, cnt, cnt0, stop;
    init_kiss4691();
    QueryPerformanceFrequency((LARGE_INTEGER*)&hz);
    stop = 5 * hz;
    QueryPerformanceCounter((LARGE_INTEGER*)&cnt0);
    for (;;) {
        for (i=0; i<CHUNK; i++) x = kiss4691();
        tot += CHUNK;
	QueryPerformanceCounter((LARGE_INTEGER*)&cnt);
        if (cnt - cnt0 >= stop) break;
    }
    sec = (double)(cnt - cnt0)/(double)hz;
    oh = overhead(tot/CHUNK);
    return (double)tot/(sec-oh);
}

double test_kiss4691_array(void)
{
    unsigned long i,x,tot=0;
    double sec, oh;
    __int64 hz, cnt, cnt0, stop;
    init_kiss4691();
    QueryPerformanceFrequency((LARGE_INTEGER*)&hz);
    stop = 5 * hz;
    QueryPerformanceCounter((LARGE_INTEGER*)&cnt0);
    for (;;) {
        array_kiss4691(CHUNK, arr);
        tot += CHUNK;
	QueryPerformanceCounter((LARGE_INTEGER*)&cnt);
        if (cnt - cnt0 >= stop) break;
    }
    sec = (double)(cnt - cnt0)/(double)hz;
    oh = overhead(tot/CHUNK);
    return (double)tot/(sec-oh);
}



int main(void)
{
    printf("Mersenne Twister 19937 speed, single (Hz): %g\n", test_mt19937());
    printf("Mersenne Twister 19937 speed, array (Hz): %g\n", test_mt19937_array());
    printf("KISS 4691 speed, single (Hz):              %g\n", test_kiss4691());
    printf("KISS 4691 speed, array (Hz):              %g\n", test_kiss4691_array());
    return 0;
}



